@@ -1,7 +1,8 @@
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
+/* exported PlacesManager */
 
-const { Gio, GLib, Shell } = imports.gi;
-const Signals = imports.signals;
+const {Gio, GLib, Shell} = imports.gi;
+const {EventEmitter} = imports.misc.signals;
 
 const ExtensionUtils = imports.misc.extensionUtils;
 const Main = imports.ui.main;
@@ -22,8 +23,10 @@ const Hostname1Iface = '<node> \
 </node>';
 const Hostname1 = Gio.DBusProxy.makeProxyWrapper(Hostname1Iface);
 
-class PlaceInfo {
+class PlaceInfo extends EventEmitter {
     constructor(...params) {
+        super();
+
         this._init(...params);
     }
 
@@ -31,7 +34,7 @@ class PlaceInfo {
         this.kind = kind;
         this.file = file;
         this.name = name || this._getFileName();
-        this.icon = icon ? new Gio.ThemedIcon({ name: icon }) : this.getIcon();
+        this.icon = icon ? new Gio.ThemedIcon({name: icon}) : this.getIcon();
     }
 
     destroy() {
@@ -94,16 +97,16 @@ class PlaceInfo {
         // icon from the query info above
         switch (this.kind) {
         case 'network':
-            return new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
+            return new Gio.ThemedIcon({name: 'folder-remote-symbolic'});
         case 'devices':
-            return new Gio.ThemedIcon({ name: 'drive-harddisk-symbolic' });
+            return new Gio.ThemedIcon({name: 'drive-harddisk-symbolic'});
         case 'special':
         case 'bookmarks':
         default:
             if (!this.file.is_native())
-                return new Gio.ThemedIcon({ name: 'folder-remote-symbolic' });
+                return new Gio.ThemedIcon({name: 'folder-remote-symbolic'});
             else
-                return new Gio.ThemedIcon({ name: 'folder-symbolic' });
+                return new Gio.ThemedIcon({name: 'folder-symbolic'});
         }
     }
 
@@ -118,7 +121,6 @@ class PlaceInfo {
         }
     }
 }
-Signals.addSignalMethods(PlaceInfo.prototype);
 
 class RootInfo extends PlaceInfo {
     _init() {
@@ -138,7 +140,7 @@ class RootInfo extends PlaceInfo {
     }
 
     getIcon() {
-        return new Gio.ThemedIcon({ name: 'drive-harddisk-symbolic' });
+        return new Gio.ThemedIcon({name: 'drive-harddisk-symbolic'});
     }
 
     _propertiesChanged(proxy) {
@@ -246,8 +248,10 @@ const DEFAULT_DIRECTORIES = [
     GLib.UserDirectory.DIRECTORY_VIDEOS,
 ];
 
-var PlacesManager = class {
+var PlacesManager = class extends EventEmitter {
     constructor() {
+        super();
+
         this._places = {
             special: [],
             devices: [],
@@ -255,7 +259,7 @@ var PlacesManager = class {
             network: [],
         };
 
-        this._settings = new Gio.Settings({ schema_id: BACKGROUND_SCHEMA });
+        this._settings = new Gio.Settings({schema_id: BACKGROUND_SCHEMA});
         this._showDesktopIconsChangedId = this._settings.connect(
             'changed::show-desktop-icons', this._updateSpecials.bind(this));
         this._updateSpecials();
@@ -543,4 +547,3 @@ var PlacesManager = class {
         return this._places[kind];
     }
 };
-Signals.addSignalMethods(PlacesManager.prototype);
